/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.

        ADDI-DATA GmbH
        Dieselstrasse 3
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You shoud find the complete GPL in the COPYING file accompanying
    this source code.
* @endverbatim
*/

#include "apci2032-private.h"
#include <apci2032-kapi.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <linux/sysfs.h>
#endif


//BEGIN JK 06.12.2004: Module informatIon
MODULE_LICENSE("GPL");
MODULE_AUTHOR("ADDI-DATA GmbH <info@addi-data.com>");
MODULE_DESCRIPTION("APCI-2032 IOCTL driver");

EXPORT_SYMBOL(apci2032_get_lock);

EXPORT_NO_SYMBOLS;

#ifndef __devinit
#define __devinit
#define __devexit
#define __devinitdata
#define __devexit_p
#endif

spinlock_t * apci2032_get_lock(struct pci_dev *pdev)
{
        return &(APCI2032_PRIVDATA(pdev)->lock);
}
//------------------------------------------------------------------------------

atomic_t apci2032_count = ATOMIC_INIT(0);
unsigned int apci2032_majornumber = 0;

//------------------------------------------------------------------------------
static int __devinit apci2032_probe_one(struct pci_dev *dev,const struct pci_device_id *ent);
static void __devexit apci2032_remove_one(struct pci_dev *dev);

//------------------------------------------------------------------------------
/** The ID table is an array of struct pci_device_id ending with a all-zero entry.
Each entry consists of:

	- vendor, device	Vendor and device ID to match (or PCI_ANY_ID)
	- subvendor,	Subsystem vendor and device ID to match (or PCI_ANY_ID)
	- subdevice class,		Device class to match. The class_mask tells which bits
	- class_mask	of the class are honored during the comparison.
	- driver_data	Data private to the driver.
*/

static struct pci_device_id apci2032_pci_tbl[] __devinitdata = {
	{
		APCI2032_BOARD_VENDOR_ID,
		APCI2032_BOARD_DEVICE_ID,
		PCI_ANY_ID,
		PCI_ANY_ID,
		0,
		0,
		0
	},

	{ 0 },	/* terminate list */
};

MODULE_DEVICE_TABLE (pci, apci2032_pci_tbl);

//----------------------------------------------------------------------------------------------------------------------
static struct pci_driver apci2032_pci_driver = {
	.name		= __DRIVER_NAME,
	.probe		= apci2032_probe_one,
	.remove		= __devexit_p(apci2032_remove_one),
	.id_table	= apci2032_pci_tbl,
};
//----------------------------------------------------------------------------------------------------------------------

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
		static struct class_simple * apci2032_class = NULL;
	#else
		static struct class * apci2032_class = NULL;
	#endif
#endif

/* Prototypes of the used functions
 */
int APCI2032_open   (struct inode *inode, struct file *filp);
int APCI2032_release(struct inode *inode, struct file *filp);
int APCI2032_ioctl  (struct        inode *inode,
                     struct        file *filp,
		     unsigned int  cmd,
		     unsigned long arg);



//------------------------------------------------------------------------------
static struct file_operations apci2032_fops =
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
	.ioctl				= apci2032_ioctl_lookup,
#else
	.unlocked_ioctl		= apci2032_ioctl_lookup,
#endif
	.open		= apci2032_open_lookup,
	.release	= apci2032_release_lookup,
	.fasync		= apci2032_fasync,
};

//------------------------------------------------------------------------------
/** when module is unloaded, stop all board activities (cf interrupt)*/
static void apci2032_stop_board(struct pci_dev * pdev)
{

}


//------------------------------------------------------------------------------
/** called when a new card is detected.
 *
 * Historically - in former ADDI-DATA drivers data about a board has been stored
 * in a static structure.
 * This led to huge duplication of information since most of these information
 * are already present in the pci_dev struct.
 *
 * Now the apci2032_str_BoardInformations contains a field "dev", a pointer to
 * the OS PCI device structure.
 *
 *  */
static int __devinit apci2032_probe_one(struct pci_dev *dev, const struct pci_device_id *ent)
{

	{
		int ret = pci_enable_device(dev);
		if (ret)
		{
			printk(KERN_ERR "%s: pci_enable_device failed\n",__DRIVER_NAME);
			return ret;
		}
	}

	/* 2.4 only : check_region is deprecated in 2.6 */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
		/* check if BAR 0 is free */
		if ( check_region(dev->resource[0].start, 4) )
		{
			printk(KERN_WARNING "%s: BAR 0 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[0].start);
			return -EBUSY;
		}
		/* check if BAR 1 is free */
		if ( check_region(dev->resource[1].start, 32) )
		{
			printk(KERN_WARNING "%s: BAR 1 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[1].start);
			return -EBUSY;
		}
	#endif // 2.4

	/* allocate a new data structure for the board */
	{
		struct apci2032_str_BoardInformations * newboard_data = NULL;
		newboard_data = kmalloc( sizeof( struct apci2032_str_BoardInformations) , GFP_ATOMIC);
		if (!newboard_data)
		{
			printk(KERN_CRIT "Can't allocate memory for new board %s\n",pci_name(dev));
			return -ENOMEM;
		}

		/* link standard data structure to the board's private one */
		pci_set_drvdata(dev,newboard_data);
	}

	/* initialise board's private data */
	apci2032_init_priv_data(APCI2032_PRIVDATA(dev));

	{
		/* increase the global board count */
		atomic_inc(&apci2032_count);
		printk(KERN_INFO "%s: board %s managed (minor number will be %d) \n",__DRIVER_NAME, pci_name(dev), atomic_read(&apci2032_count)-1);
	}

	/* allocate BAR 0 to 2 memory regions */
	{
		int ret = pci_request_regions(dev,__DRIVER_NAME);
		if (ret)
		{
			printk(KERN_ERR "%s: pci_request_regions failed\n",__DRIVER_NAME);
			/* free all allocated ressources here*/
			kfree(APCI2032_PRIVDATA(dev));
			return ret;
		}
	}

	/* create /proc */
	apci2032_proc_create_device(dev, atomic_read(&apci2032_count)-1);

	apci2032_register_interrupt(dev);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
        while(1)
        {
            #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
				struct class_device *cdev;
			#else
				struct device *cdev;
			#endif

				int minor = (atomic_read(&apci2032_count)-1);

                /* don't execute if class not exists */
                if (IS_ERR(apci2032_class))
                        break;

			#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
				cdev = class_device_create (apci2032_class, NULL, MKDEV(apci2032_majornumber, minor), NULL, "%s_%d", __DRIVER_NAME, minor);
			#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
				cdev = device_create (apci2032_class, NULL, MKDEV(apci2032_majornumber, minor), "%s_%d", __DRIVER_NAME, minor);
			#else
				cdev = device_create (apci2032_class, NULL, MKDEV(apci2032_majornumber, minor), NULL, "%s_%d", __DRIVER_NAME, minor);
			#endif

                if (IS_ERR(cdev))
                {
                        printk (KERN_WARNING "%s: class_device_create error\n", __DRIVER_NAME);
                }
                break;
        }
#endif

	return 0;
}


//------------------------------------------------------------------------------
/** This function registers the driver with register_chrdev.
 *
 * @todo For kernel 2.6, use udev
 *  */

static int apci2032_register(void)
{

	//Registration of driver
	apci2032_majornumber = register_chrdev(0, __DRIVER_NAME, &apci2032_fops);

	if (apci2032_majornumber < 0)
	{
		printk (KERN_ERR "%s: register_chrdev returned %d ... aborting\n",__DRIVER_NAME,apci2032_majornumber);
		return -ENODEV;
	}
	return 0;
}

//------------------------------------------------------------------------------
/** Called when module loads. */
static int __init apci2032_init(void)
{

	/* 2.4 only : check_region is deprecated in 2.6 */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
		/* Detection if an PCI Bus is present */
		if(!pci_present())
		{
			printk(KERN_WARNING "%s: NO PCI BUS.\n", __DRIVER_NAME);
			return -ENODEV;
		}
	#endif // 2.4

	if( apci2032_register() )
	{
		return -ENODEV;
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	apci2032_class = ADDI_CLASS_CREATE(THIS_MODULE, __DRIVER_NAME);
	if (IS_ERR(apci2032_class))
	{
		printk (KERN_WARNING "%s: class_create error\n",__DRIVER_NAME );
	}
#endif

	/* registred, now create root /proc entry */
	apci2032_proc_init();

	printk(KERN_INFO "%s: loaded\n",__DRIVER_NAME);

	/* now, subscribe to PCI bus subsystem  */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
	/** @note The module will load anyway even if the call to pci_module_init() fails */
	if ( pci_module_init (&apci2032_pci_driver) != 0)
#else
	if ( pci_register_driver (&apci2032_pci_driver) != 0)
#endif
	{
		printk("%s: no device found\n",__DRIVER_NAME);
		apci2032_pci_driver.name = "";
	}

	return 0;
}

//-------------------------------------------------------------------
/** a card is removed */
static void __devexit apci2032_remove_one(struct pci_dev *dev)
{

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	while(1)
	{
		int minor = (atomic_read(&apci2032_count)-1);

		/* don't execute if class not exists */
		if(IS_ERR(apci2032_class))
				break;

		#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
			class_device_destroy (apci2032_class, MKDEV(apci2032_majornumber, minor));
		#else
			device_destroy (apci2032_class, MKDEV(apci2032_majornumber, minor));
		#endif

		break;
	}
#endif

	/* stop board activities */
	apci2032_stop_board(dev);

	/* now the board is stopped, deregistering interrupt should be safe*/
	apci2032_deregister_interrupt(dev);

	/* do system-dependant thing we don't know of */
	pci_disable_device(dev);

	/* deallocate BAR 0 to 2 memory regions */
	pci_release_regions(dev);

	/*free private device data */
	kfree(APCI2032_PRIVDATA(dev));

	/* delete associated /proc entry */
	apci2032_proc_release_device(dev);

	/* decrement global board count*/
	atomic_dec(&apci2032_count);

	printk("%s: board %s unloaded \n",__DRIVER_NAME,pci_name(dev));

}

//------------------------------------------------------------------------------
/** This function unregisters the driver with unregister_chrdev.
 *
 * @todo For kernel 2.6, use udev
 *  */

static void apci2032_unregister(void)
{
	unregister_chrdev(apci2032_majornumber, __DRIVER_NAME);
}

//-------------------------------------------------------------------
/** Called when module is unloaded. */
static void __exit apci2032_exit(void)
{

	/* Cleans the module from the kernel */
	if (apci2032_pci_driver.name[0])
	{
		pci_unregister_driver (&apci2032_pci_driver);
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	/* don't execute if class not exists */
	if ( !IS_ERR(apci2032_class) )
		class_destroy (apci2032_class);
#endif

	apci2032_unregister();
	apci2032_proc_release();
}
//------------------------------------------------------------------------------
module_exit(apci2032_exit);
module_init(apci2032_init);
