#ifndef __APCI2032_IOMAP_H__
#define __APCI2032_IOMAP_H__

/* registers  */
#define APCI2032_DIGITAL_OUTPUT_REG 			0x00 /**< digital IO */
#define APCI2032_INTERRUPT_CONTROL_REG 			0x01 /**< control diagnostic interrupts  */
#define APCI2032_INTERRUPT_STATUS_REG 			0x02 /**< status of diagnostic */
#define APCI2032_IRQ_STATUS_REG 				0x03 /**< state of the IRQ line */


#endif // __APCI2032_IOMAP_H__
